AU = 1.49597870691e13                   # Astronomical unit [cgs]
au = 1.49597870691e13                   # Astronomical unit [cgs], as of IAU decree
k = 1.3806503e-16                       # Boltzmann's constant [cgs]
h = 6.62606876e-27                      # Planck's constant [cgs]
c = 2.99792458e10                       # Speed of light [cgs]
micron = 1e-4                           # Micrometer [cgs]
jy = 1e-23                              # Jansky [cgs]
Msun = 1.989e33                         # Solar Mass [cgs]
Rsun = 6.960e10                         # Solar Radius [cgs]
sigma = 5.671e-5                        # Boltzmann Constant [cgs]
G = 6.673e-8                            # Gravitational constant [cgs]
pc = 3.08e18                            # Parsec [cm]
amu = 1.66053886e-24                    # Atomic Mass Unit [g]

min = 60
hour = 3600
day = 86400
year = 3.15569e7                        # google

Rearth = 6.37814e8                      # google
Mearth = 5.97219e27                     # google
rhoEarth= 5.51 							# google
Lsun = 3.839e33                         # google
Psun = 24.47 * day 						# google
Tsun = 5780								# google

Rjup= 69911e5							# google
Mjup= 1.898e30							# google